@@ -16,8 +16,8 @@
 *
 * Authors:
 *
-*  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
-*  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+*  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+*  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
 *
 * Hosted on http://github.com/sahib/rmlint
 **/
@@ -40,10 +40,13 @@ typedef struct RmPath {
     guint idx;
 
     /* whether path was tagged as preferred path */
-    bool is_prefd;
+    bool is_prefd : 1;
 
     /* whether to treat all files under path as one filesystem */
-    bool treat_as_single_vol;
+    bool treat_as_single_vol : 1;
+
+    /* wether calling realpath() worked on this path */
+    bool realpath_worked : 1;
 } RmPath;
 
 /* Storage struct for all options settable in cmdline. */
@@ -89,7 +92,7 @@ typedef struct RmCfg {
     gboolean replay;
     gboolean read_stdin;
     gboolean read_stdin0;
-    gboolean no_backup;
+    gboolean backup;
 
     int permissions;
 
