@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -277,32 +277,119 @@ static RmOff rm_cmd_size_string_to_bytes(const char *size_spec, GError **error)
         return 0;
     }
 
-    char *format = NULL;
-    long double decimal = strtold(size_spec, &format);
+    // We have a small issue here:
+    // 1) strold can not parse all of a guin64.
+    // 2) strtoull can only parse integers.
+    //
+    // We "solve" this by parsing with strtoull until the dot,
+    // and continue with strtold after. If we hit an overflow
+    // during multiplication later we warn the user and abort.
 
-    if(decimal == 0 && format == size_spec) {
-        g_set_error(error, RM_ERROR_QUARK, 0, _("This does not look like a number"));
-        return 0;
-    } else if(decimal < 0) {
-        g_set_error(error, RM_ERROR_QUARK, 0, _("Negativ sizes are no good idea"));
-        return 0;
-    } else if(*format) {
-        format = g_strstrip(format);
-    } else {
-        return round(decimal);
-    }
+    // Copy and strip the string:
+    char size_spec_copy[512] = {0};
+    memset(size_spec_copy, 0, sizeof(size_spec_copy));
+    strncpy(size_spec_copy, size_spec, sizeof(size_spec_copy)-1);
+    char *size_spec_stripped = g_strstrip(size_spec_copy);
 
-    FormatSpec key = {.id = format};
-    FormatSpec *found = bsearch(&key, SIZE_FORMAT_TABLE, SIZE_FORMAT_TABLE_N,
-                                sizeof(FormatSpec), rm_cmd_compare_spec_elem);
-
-    if(found != NULL) {
-        /* No overflow check */
-        return decimal * pow(found->base, found->exponent);
-    } else {
-        g_set_error(error, RM_ERROR_QUARK, 0, _("Given format specifier not found"));
+    if(*size_spec_stripped == 0) {
+        g_set_error(error, RM_ERROR_QUARK, 0, _("Size is empty"));
         return 0;
     }
+
+    if(*size_spec_stripped == '-') {
+        g_set_error(error, RM_ERROR_QUARK, 0, _("Negative sizes are no good idea"));
+        return 0;
+    }
+
+    // Actual number before the dot.
+    RmOff base_size = 1;
+
+    // size_factor to multiply the result with (in case of a suffix like »mb«)
+    double size_factor = 1;
+
+    // possible fraction given for a size after a dot.
+    double fraction = 0;
+
+    char *size_format = size_spec_stripped;
+    for(; *size_format && (*size_format == '.' || g_ascii_isdigit(*size_format)); size_format++);
+
+    // Set to true when either a format suffix or a fraction was encountered.
+    bool need_multiply = false;
+
+    if(*size_format != 0) {
+        need_multiply = true;
+        FormatSpec key = {.id = size_format};
+        FormatSpec *found = bsearch(
+            &key,
+            SIZE_FORMAT_TABLE,
+            SIZE_FORMAT_TABLE_N,
+            sizeof(FormatSpec),
+            rm_cmd_compare_spec_elem
+        );
+
+        if(found != NULL) {
+            size_factor *= pow(found->base, found->exponent);
+        } else {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("Given size_format specifier not found"));
+            return 0;
+        }
+
+        // Set to zero to separate it from the after-dot part.
+        *size_format = 0;
+    }
+
+    char *dot = strchr(size_spec_stripped, '.');
+    if(dot != NULL) {
+        need_multiply = true;
+
+        // Parse with strtoull since we only accept numbers after the dot.
+        // (not 1e10 or negative number like strtod would support them)
+        char *first_error = NULL;
+        RmOff fraction_num = strtoull(&dot[1], &first_error, 10);
+        if(first_error != NULL && *first_error != 0) {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("Failed to parse size fraction"));
+            return 0;
+        }
+
+        if(fraction_num == ULONG_MAX && errno == ERANGE) {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("Fraction is too big for uint64"));
+            return 0;
+        }
+
+        // Make sure number parsing doesn't see the dot.
+        *dot = 0;
+
+        // Convert the integer (e.g. 523) to a fraction (e.g. 0.523).
+        fraction = ((double)fraction_num) / pow(10, 1 + ((int)log10(fraction_num)));
+    }
+
+    if(*size_spec_stripped != 0) {
+        char *first_error = NULL;
+        base_size = strtoull(size_spec_stripped, &first_error, 10);
+        if(first_error != NULL && *first_error != 0) {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("This does not look like a number"));
+            return 0;
+        }
+
+        if(base_size == ULONG_MAX && errno == ERANGE) {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("Size is too big for uint64"));
+            return 0;
+        }
+    }
+
+    RmOff result = base_size;
+    if(need_multiply) {
+        // Only multiply if we really have to.
+        result = (result + fraction) * size_factor;
+
+        // Check if an overflow happened during multiplication.
+        if(result < base_size) {
+            g_set_error(error, RM_ERROR_QUARK, 0, _("Size factor would overflow size (max. 2**64 allowed)"));
+            return 0;
+        }
+    }
+
+    return result;
 }
 
 /* Size spec parsing implemented by qitta (http://github.com/qitta)
@@ -323,19 +410,20 @@ static gboolean rm_cmd_size_range_string_to_bytes(const char *range_spec, RmOff
 
     if(split[0] != NULL) {
         *min = rm_cmd_size_string_to_bytes(split[0], error);
-    }
 
-    if(split[1] != NULL && *error == NULL) {
-        *max = rm_cmd_size_string_to_bytes(split[1], error);
+        if(split[1] != NULL && *error == NULL) {
+            *max = rm_cmd_size_string_to_bytes(split[1], error);
+        }
     }
 
     g_strfreev(split);
 
     if(*error == NULL && *max < *min) {
         g_set_error(error, RM_ERROR_QUARK, 0, _("Max is smaller than min"));
+        return false;
     }
 
-    return (*error == NULL);
+    return true;
 }
 
 static gboolean rm_cmd_parse_limit_sizes(_UNUSED const char *option_name,
@@ -387,6 +475,7 @@ static bool rm_cmd_read_paths_from_stdin(RmSession *session, bool is_prefd,
             if (path_buf[path_len - 1] == delim) {
                 path_buf[path_len - 1] = 0;
             }
+
             all_paths_read &= rm_cfg_add_path(session->cfg, is_prefd, path_buf);
         }
     }
@@ -538,6 +627,41 @@ static void rm_cmd_parse_clamp_option(RmSession *session, const char *string,
     }
 }
 
+static gboolean rm_cmd_parse_partial_hidden(_UNUSED const char *option_name,
+                                            _UNUSED const gchar *count,
+                                            RmSession *session, _UNUSED GError **error) {
+    RmCfg *cfg = session->cfg;
+    cfg->ignore_hidden = false;
+    cfg->partial_hidden = true;
+
+    return true;
+}
+
+static gboolean rm_cmd_parse_merge_directories(_UNUSED const char *option_name,
+                                               _UNUSED const gchar *_,
+                                               RmSession *session,
+                                               _UNUSED GError **error) {
+    RmCfg *cfg = session->cfg;
+    cfg->merge_directories = true;
+
+    /* Pull in some options for convenience,
+     * duplicate dir detection works better with them.
+     *
+     * They may be disabled explicitly though.
+     */
+    cfg->follow_symlinks = false;
+    cfg->see_symlinks = true;
+    rm_cmd_parse_partial_hidden(NULL, NULL, session, error);
+
+    cfg->find_hardlinked_dupes = true;
+
+    /* Keep RmFiles after shredder. */
+    cfg->cache_file_structs = true;
+
+    return true;
+}
+
+
 /* parse comma-separated strong of lint types and set cfg accordingly */
 typedef struct RmLintTypeOption {
     const char **names;
@@ -665,9 +789,7 @@ static gboolean rm_cmd_parse_lint_types(_UNUSED const char *option_name,
     }
 
     if(cfg->merge_directories) {
-        cfg->ignore_hidden = false;
-        cfg->find_hardlinked_dupes = true;
-        cfg->cache_file_structs = true;
+        rm_cmd_parse_merge_directories(NULL, NULL, session, NULL);
     }
 
     /* clean up */
@@ -954,16 +1076,6 @@ static gboolean rm_cmd_parse_less_paranoid(_UNUSED const char *option_name,
     return true;
 }
 
-static gboolean rm_cmd_parse_partial_hidden(_UNUSED const char *option_name,
-                                            _UNUSED const gchar *count,
-                                            RmSession *session, _UNUSED GError **error) {
-    RmCfg *cfg = session->cfg;
-    cfg->ignore_hidden = false;
-    cfg->partial_hidden = true;
-
-    return true;
-}
-
 static gboolean rm_cmd_parse_see_symlinks(_UNUSED const char *option_name,
                                           _UNUSED const gchar *count, RmSession *session,
                                           _UNUSED GError **error) {
@@ -984,6 +1096,16 @@ static gboolean rm_cmd_parse_follow_symlinks(_UNUSED const char *option_name,
     return true;
 }
 
+static gboolean rm_cmd_parse_no_follow_symlinks(_UNUSED const char *option_name,
+                                             _UNUSED const gchar *count,
+                                             RmSession *session, _UNUSED GError **error) {
+    RmCfg *cfg = session->cfg;
+    cfg->see_symlinks = false;
+    cfg->follow_symlinks = false;
+
+    return true;
+}
+
 static gboolean rm_cmd_parse_no_partial_hidden(_UNUSED const char *option_name,
                                                _UNUSED const gchar *count,
                                                RmSession *session,
@@ -994,29 +1116,6 @@ static gboolean rm_cmd_parse_no_partial_hidden(_UNUSED const char *option_name,
 
     return true;
 }
-
-static gboolean rm_cmd_parse_merge_directories(_UNUSED const char *option_name,
-                                               _UNUSED const gchar *_,
-                                               RmSession *session,
-                                               _UNUSED GError **error) {
-    RmCfg *cfg = session->cfg;
-    cfg->merge_directories = true;
-
-    /* Pull in some options for convenience,
-     * duplicate dir detection works better with them.
-     *
-     * They may be disabled explicitly though.
-     */
-    cfg->follow_symlinks = false;
-    cfg->see_symlinks = true;
-    rm_cmd_parse_partial_hidden(NULL, NULL, session, error);
-
-    /* Keep RmFiles after shredder. */
-    cfg->cache_file_structs = true;
-
-    return true;
-}
-
 static gboolean rm_cmd_parse_hidden(_UNUSED const char *option_name,
                                                _UNUSED const gchar *_,
                                                RmSession *session,
@@ -1222,9 +1321,7 @@ static bool rm_cmd_set_paths(RmSession *session, char **paths) {
         /* option '-' means read paths from stdin */
         all_paths_valid &=
             rm_cmd_read_paths_from_stdin(session, stdin_paths_preferred, cfg->read_stdin0);
-    }
-
-    if(g_slist_length(cfg->paths) == 0 && all_paths_valid) {
+    } else if(g_slist_length(cfg->paths) == 0 && all_paths_valid) {
         /* Still no path set? - use `pwd` */
         rm_cfg_add_path(session->cfg, is_prefd, cfg->iwd);
     }
@@ -1323,7 +1420,7 @@ bool rm_cmd_parse_args(int argc, char **argv, RmSession *session) {
         {"no-with-color"            , 'W'  , DISABLE   , G_OPTION_ARG_NONE      , &cfg->with_color               , _("Be not that colorful")                                                 , NULL}     ,
         {"hidden"                   , 'r'  , EMPTY     , G_OPTION_ARG_CALLBACK  , FUNC(hidden)                   , _("Find hidden files")                                                    , NULL}     ,
         {"followlinks"              , 'f'  , EMPTY     , G_OPTION_ARG_CALLBACK  , FUNC(follow_symlinks)          , _("Follow symlinks")                                                      , NULL}     ,
-        {"no-followlinks"           , 'F'  , DISABLE   , G_OPTION_ARG_NONE      , &cfg->follow_symlinks          , _("Ignore symlinks")                                                      , NULL}     ,
+        {"no-followlinks"           , 'F'  , EMPTY     , G_OPTION_ARG_CALLBACK  , FUNC(no_follow_symlinks)       , _("Ignore symlinks")                                                      , NULL}     ,
         {"paranoid"                 , 'p'  , EMPTY     , G_OPTION_ARG_CALLBACK  , FUNC(paranoid)                 , _("Use more paranoid hashing")                                            , NULL}     ,
         {"no-crossdev"              , 'x'  , DISABLE   , G_OPTION_ARG_NONE      , &cfg->crossdev                 , _("Do not cross mountpoints")                                             , NULL}     ,
         {"keep-all-tagged"          , 'k'  , 0         , G_OPTION_ARG_NONE      , &cfg->keep_all_tagged          , _("Keep all tagged files")                                                , NULL}     ,
@@ -1341,7 +1438,7 @@ bool rm_cmd_parse_args(int argc, char **argv, RmSession *session) {
         {"partial-hidden"           , 0    , EMPTY     , G_OPTION_ARG_CALLBACK  , FUNC(partial_hidden)           , _("Find hidden files in duplicate folders only")                          , NULL}     ,
         {"mtime-window"             , 'Z'  , 0         , G_OPTION_ARG_DOUBLE    , &cfg->mtime_window             , _("Consider duplicates only equal when mtime differs at max. T seconds")  , "T"}      ,
         {"stdin0"                   , '0'  , 0         , G_OPTION_ARG_NONE      , &cfg->read_stdin0              , _("Read null-separated file list from stdin")                             , NULL}     ,
-        {"no-backup"                , 0    , 0         , G_OPTION_ARG_NONE      , &cfg->no_backup                , _("Do not create backups of previous result files")                       , NULL}     ,
+        {"backup"                   , 0    , 0         , G_OPTION_ARG_NONE      , &cfg->backup                   , _("Do create backups of previous result files")                           , NULL}     ,
 
         /* COW filesystem deduplication support */
         {"dedupe"                   , 0    , 0         , G_OPTION_ARG_NONE      , &cfg->dedupe                   , _("Dedupe matching extents from source to dest (if filesystem supports)") , NULL}     ,
@@ -1380,8 +1477,8 @@ bool rm_cmd_parse_args(int argc, char **argv, RmSession *session) {
     };
 
     const GOptionEntry unusual_option_entries[] = {
-        {"clamp-low"              , 'q' , HIDDEN           , G_OPTION_ARG_CALLBACK , FUNC(clamp_low)              , "Limit lower reading barrier"                                 , "P"}    ,
-        {"clamp-top"              , 'Q' , HIDDEN           , G_OPTION_ARG_CALLBACK , FUNC(clamp_top)              , "Limit upper reading barrier"                                 , "P"}    ,
+        {"clamp-low"              , 'q' , 0                , G_OPTION_ARG_CALLBACK , FUNC(clamp_low)              , "Limit lower reading barrier"                                 , "P"}    ,
+        {"clamp-top"              , 'Q' , 0                , G_OPTION_ARG_CALLBACK , FUNC(clamp_top)              , "Limit upper reading barrier"                                 , "P"}    ,
         {"limit-mem"              , 'u' , HIDDEN           , G_OPTION_ARG_CALLBACK , FUNC(limit_mem)              , "Specify max. memory usage target"                            , "S"}    ,
         {"read-buffer-len"        , 0   , HIDDEN           , G_OPTION_ARG_CALLBACK , FUNC(read_buf_len)           , "Specify read buffer length in bytes"                         , "S"}    ,
         {"sweep-size"             , 0   , HIDDEN           , G_OPTION_ARG_CALLBACK , FUNC(sweep_size)             , "Specify max. bytes per pass when scanning disks"             , "S"}    ,
@@ -1618,11 +1715,12 @@ static int rm_cmd_replay_main(RmSession *session) {
         return EXIT_FAILURE;
     }
 
-    rm_parrot_cage_close(&cage);
+    rm_parrot_cage_flush(&cage);
     rm_fmt_flush(session->formats);
     rm_fmt_set_state(session->formats, RM_PROGRESS_STATE_PRE_SHUTDOWN);
     rm_fmt_set_state(session->formats, RM_PROGRESS_STATE_SUMMARY);
 
+    rm_parrot_cage_close(&cage);
     return EXIT_SUCCESS;
 }
 
@@ -1673,8 +1771,8 @@ int rm_cmd_main(RmSession *session) {
             return EXIT_FAILURE;
         }
 
-        RmTreeMerger *t = session->dir_merger = rm_tm_new(session);
-        if(!t) {
+        session->dir_merger = rm_tm_new(session);
+        if(!session->dir_merger) {
             rm_log_error_line(_("Failed to complete setup for merging directories"));
             return EXIT_FAILURE;
         }
@@ -1692,6 +1790,7 @@ int rm_cmd_main(RmSession *session) {
         RmPath *a = cfg->paths->data;
         g_assert(cfg->paths->next);
         RmPath *b = cfg->paths->next->data;
+
         switch(rm_util_link_type(a->path, b->path)) {
         case RM_LINK_HARDLINK:
         case RM_LINK_REFLINK:
@@ -1725,6 +1824,7 @@ int rm_cmd_main(RmSession *session) {
 
     if(cfg->merge_directories) {
         rm_fmt_set_state(session->formats, RM_PROGRESS_STATE_MERGE);
+        rm_tm_set_callback(session->dir_merger, (RmTreeMergeOutputFunc)rm_shred_output_tm_results, session);
         rm_tm_finish(session->dir_merger);
     }
 
