@@ -16,8 +16,8 @@
 *
 * Authors:
 *
-*  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
-*  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+*  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+*  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
 *
 * Hosted on http://github.com/sahib/rmlint
 *
@@ -76,6 +76,11 @@ typedef enum RmLintType {
      * This is mainly useful for caching.
      */
     RM_LINT_TYPE_UNIQUE_FILE,
+
+    /* Special type for files that are outputted as part of treemerge.
+     * They land still in the json output with this type.
+     */
+    RM_LINT_TYPE_PART_OF_DIRECTORY,
 } RmLintType;
 
 struct RmSession;
@@ -95,6 +100,8 @@ typedef guint16 RmPatternBitmask;
 #define RM_PATTERN_SET_CACHED(mask, idx, match) \
     ((*mask) |= ((!!match << idx) | (1 << (idx + RM_PATTERN_N_MAX))))
 
+struct RmDirectory;
+
 /**
  * RmFile structure; used by pretty much all rmlint modules.
  */
@@ -252,6 +259,16 @@ typedef struct RmFile {
      * */
     RmPatternBitmask pattern_bitmask_path;
     RmPatternBitmask pattern_bitmask_basename;
+
+    /* Parent directory.
+     * Only filled if type is RM_LINT_TYPE_PART_OF_DIRECTORY.
+     */
+    struct RmDirectory *parent_dir;
+
+    /* Number of children this file has.
+     * Only filled if type is RM_LINT_TYPE_PART_OF_DIRECTORY.
+     * */
+    size_t n_children;
 } RmFile;
 
 /* Defines a path variable containing the file's path */
@@ -347,6 +364,13 @@ void rm_file_set_path(RmFile *file, char *path);
  */
 void rm_file_build_path(RmFile *file, char *buf);
 
+/**
+ * @brief Make a copy of a file.
+ *
+ * Free with rm_file_destroy.
+ */
+RmFile *rm_file_copy(RmFile *file);
+
 /**
  * @brief Compare basenames of two files
  * @retval true if basenames match.
