@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  */
@@ -622,7 +622,8 @@ static RmOff rm_pp_handler_other_lint(const RmSession *session) {
 
             num_handled++;
 
-            rm_fmt_write(file, session->formats, -1);
+            file->twin_count = -1;
+            rm_fmt_write(file, session->formats);
         }
 
         if(!session->cfg->cache_file_structs) {
