@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -159,8 +159,7 @@ static void rm_traverse_file(RmTravSession *trav_session, RmStat *statp, char *p
         } else {
             RmOff file_size = statp->st_size;
             if(!cfg->limits_specified ||
-               ((cfg->minsize == (RmOff)-1 || cfg->minsize <= file_size) &&
-                (cfg->maxsize == (RmOff)-1 || file_size <= cfg->maxsize))) {
+               ((cfg->minsize <= file_size) && (file_size <= cfg->maxsize))) {
                 if(rm_mounts_is_evil(trav_session->session->mounts, statp->st_dev) ==
                    false) {
                     file_type = RM_LINT_TYPE_DUPE_CANDIDATE;
@@ -179,7 +178,7 @@ static void rm_traverse_file(RmTravSession *trav_session, RmStat *statp, char *p
     if(is_symlink && cfg->follow_symlinks) {
         char *new_path_buf = g_malloc0(PATH_MAX + 1);
         if(readlink(path, new_path_buf, PATH_MAX) == -1) {
-            rm_log_warning_line("failed to follow symbolic link of %s: %s", path, g_strerror(errno));
+            rm_log_debug_line("failed to follow symbolic link of %s: %s", path, g_strerror(errno));
             g_free(new_path_buf);
             return;
         }
@@ -314,7 +313,6 @@ static void rm_traverse_directory(RmTravBuffer *buffer, RmTravSession *trav_sess
     bool have_open_emptydirs = false;
     bool clear_emptydir_flags = false;
     bool next_is_symlink = false;
-    bool symlink_message_delivered = false;
 
     memset(is_emptydir, 0, sizeof(is_emptydir) - 1);
     memset(is_hidden, 0, sizeof(is_hidden) - 1);
@@ -413,32 +411,22 @@ static void rm_traverse_directory(RmTravBuffer *buffer, RmTravSession *trav_sess
             case FTS_SL:                     /* symbolic link */
                 clear_emptydir_flags = true; /* current dir not empty */
                 if(!cfg->follow_symlinks) {
-                    if(p->fts_level != 0 && !symlink_message_delivered) {
-                        rm_log_debug_line(
-                            "Not following symlink %s because of cfg\n"
-                            "\t(further symlink messages suppressed for this cmdline "
-                            "path)",
-                            p->fts_path);
-                        symlink_message_delivered = TRUE;
+                    bool is_badlink = false;
+                    if(access(p->fts_path, R_OK) == -1 && errno == ENOENT) {
+                        is_badlink = true;
                     }
 
-                    if(access(p->fts_path, R_OK) == -1 && errno == ENOENT) {
-                        /* Oops, that's a badlink. */
-                        if(cfg->find_badlinks) {
-                            ADD_FILE(RM_LINT_TYPE_BADLINK, false);
-                        }
+                    if(is_badlink && cfg->find_badlinks) {
+                        ADD_FILE(RM_LINT_TYPE_BADLINK, false);
                     } else if(cfg->see_symlinks) {
+                        /* NOTE: bad links are also counted as duplicates
+                         *       when -T df,dd (for example) is used.
+                         *       They can serve as input for the treemerge
+                         *       algorithm which might fail when missing.
+                         */
                         ADD_FILE(RM_LINT_TYPE_UNKNOWN, true);
                     }
                 } else {
-                    if(!symlink_message_delivered) {
-                        rm_log_debug_line(
-                            "Following symlink %s\n"
-                            "\t(further symlink messages suppressed for this cmdline "
-                            "path)",
-                            p->fts_path);
-                        symlink_message_delivered = TRUE;
-                    }
                     next_is_symlink = true;
                     fts_set(ftsp, p, FTS_FOLLOW); /* do recurse */
                 }
@@ -482,7 +470,6 @@ static void rm_traverse_directory(RmTravBuffer *buffer, RmTravSession *trav_sess
 
     fts_close(ftsp);
 
-    // rm_file_list_insert_queue(, session);
     rm_fmt_set_state(session->formats, RM_PROGRESS_STATE_TRAVERSE);
 
 done:
@@ -511,15 +498,22 @@ void rm_traverse_tree(RmSession *session) {
         RmPath *rmpath = iter->data;
         RmTravBuffer *buffer = rm_trav_buffer_new(session, rmpath);
 
-        if(S_ISREG(buffer->stat_buf.st_mode)) {
-            /* Append normal paths directly */
-            bool is_hidden = false;
+        /* Append normal paths directly */
+        bool is_hidden = false;
 
-            /* The is_hidden information is only needed for --partial-hidden */
-            if(cfg->partial_hidden) {
-                is_hidden = rm_util_path_is_hidden(rmpath->path);
-            }
+        /* The is_hidden information is only needed for --partial-hidden */
+        if(cfg->partial_hidden) {
+            is_hidden = rm_util_path_is_hidden(rmpath->path);
+        }
 
+        if(S_ISLNK(buffer->stat_buf.st_mode) && !rmpath->realpath_worked) {
+            /* A symlink where we could not get the actual path from
+             * (and it was given directly, e.g. by a find call)
+             */
+            rm_traverse_file(trav_session, &buffer->stat_buf, rmpath->path,
+                             rmpath->is_prefd, rmpath->idx, RM_LINT_TYPE_BADLINK, false,
+                             is_hidden, FALSE, 0);
+        } else if(S_ISREG(buffer->stat_buf.st_mode)) {
             rm_traverse_file(trav_session, &buffer->stat_buf, rmpath->path,
                              rmpath->is_prefd, rmpath->idx, RM_LINT_TYPE_UNKNOWN, false,
                              is_hidden, FALSE, 0);
