@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 # encoding: utf-8
 from nose import with_setup
+from nose.plugins.attrib import attr
 from tests.utils import *
 
 from itertools import permutations, combinations
@@ -52,6 +53,7 @@ def validate_order(data, tests):
             assert False
 
 
+@attr('slow')
 @with_setup(usual_setup_func, usual_teardown_func)
 def test_sorting():
     # create some dupes with different PATHS, names and mtimes:
