@@ -13,6 +13,9 @@ import shutil
 import shlex
 import struct
 import subprocess
+import contextlib
+import xattr
+
 
 TESTDIR_NAME = os.getenv('RM_TS_DIR') or '/tmp/rmlint-unit-testdir'
 
@@ -40,7 +43,7 @@ CKSUM_TYPES = [
 ]
 
 def runs_as_root():
-    return os.geteuid() is 0
+    return os.geteuid() == 0
 
 
 def get_env_flag(name):
@@ -265,7 +268,7 @@ def run_rmlint_pedantic(*args, **kwargs):
 
         data_skip, new_data_skip = data, new_data
 
-        if output_len is not 0:
+        if output_len != 0:
             if new_data:
                 new_data_skip = new_data[:-output_len]
 
@@ -351,3 +354,50 @@ def usual_teardown_func():
             shutil.rmtree(path=TESTDIR_NAME)
         except OSError:
             pass
+
+
+@contextlib.contextmanager
+def create_special_fs(name, fs_type='ext4'):
+    """
+    Used to create a special filesystem container in TESTDIR_NAME
+    under »name«. The type of the filesystem will be »fs_type« (as long
+    we have a »mkfs.xyz« binary for that.) This method needs root privileges.
+
+    Returns: The path of the created directory.
+    """
+    mount_path = os.path.join(TESTDIR_NAME, name)
+    device_path = mount_path + ".device"
+
+    commands = [
+        "dd if=/dev/zero of={} bs=1M count=20".format(device_path),
+        "mkdir -p {}".format(mount_path),
+        "mkfs.{} {}".format(fs_type, device_path),
+        "mount -o loop {} {}".format(device_path, mount_path),
+    ]
+
+    for command in commands:
+        subprocess.run(
+                command,
+                shell=True,
+                check=True,
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL,
+        )
+
+    try:
+        yield mount_path
+    finally:
+        # whatever happens: unmount it again.
+        # we'll get test errors in the next tests otherwise.
+        unmount_command = "umount {}".format(mount_path)
+        subprocess.run(unmount_command, shell=True, check=True)
+
+
+def must_read_xattr(path):
+    """
+    Read all extended attributes of a »path«.
+
+    NOTE: This will only work on non-tmpfs mounts.
+          See create_special_fs for a workaround.
+    """
+    return dict(xattr.xattr(os.path.join(TESTDIR_NAME, path)).items())
